package com.fjhx.mes.service.production.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.entity.product.po.ProductBomDetail;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductBomDetailService;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.material.po.MaterialPreparation;
import com.fjhx.mes.entity.production.dto.ProduceOrderSelectDto;
import com.fjhx.mes.entity.production.dto.ProductionOrderDto;
import com.fjhx.mes.entity.production.po.ProductionOrder;
import com.fjhx.mes.entity.production.po.ProductionOrderDetail;
import com.fjhx.mes.entity.production.vo.ProductionOrderDetailVo;
import com.fjhx.mes.entity.production.vo.ProductionOrderVo;
import com.fjhx.mes.entity.work.po.WorkOrder;
import com.fjhx.mes.mapper.production.ProduceOrderMapper;
import com.fjhx.mes.service.material.MaterialPreparationService;
import com.fjhx.mes.service.production.ProduceOrderDetailService;
import com.fjhx.mes.service.production.ProduceOrderService;
import com.fjhx.mes.service.work.WorkOrderService;
import com.fjhx.sale.entity.contract.po.Contract;
import com.fjhx.sale.entity.contract.po.ContractProduct;
import com.fjhx.sale.service.contract.ContractProductService;
import com.fjhx.sale.service.contract.ContractService;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 生产订单 服务实现类
 * </p>
 *
 * @since 2024-01-10
 */
@Service
public class ProduceOrderServiceImpl extends ServiceImpl<ProduceOrderMapper, ProductionOrder> implements ProduceOrderService {

    @Autowired
    private ContractService contractService;
    @Autowired
    private ContractProductService contractProductService;
    @Autowired
    private ProduceOrderDetailService produceOrderDetailService;
    @Autowired
    private ISysDeptService sysDeptService;
    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private WorkOrderService workOrderService;
    @Autowired
    private ProductBomDetailService productBomDetailService;
    @Autowired
    private MaterialPreparationService materialPreparationService;

    @Override
    public Page<ProductionOrderVo> getPage(ProduceOrderSelectDto dto) {
        IWrapper<ProductionOrder> wrapper = getWrapper();
        wrapper.orderByDesc("po", ProductionOrder::getId);

        //生产状态
        wrapper.eq(ProductionOrder::getProduceStatus, dto.getProduceStatus());
        //交期-时间范围过滤
        wrapper.ge("po", ProductionOrder::getDeliveryPeriod, dto.getStaDeliveryPeriod());
        wrapper.le("po", ProductionOrder::getDeliveryPeriod, dto.getEndDeliveryPeriod());
        //下单时间（创建时间）-时间范围过滤
        wrapper.ge("po", ProductionOrder::getCreateTime, dto.getBeginTime());
        wrapper.le("po", ProductionOrder::getCreateTime, dto.getEndTime());
        //关键字
        wrapper.keyword(dto.getKeyword(), new SqlField(ProductionOrder::getCode));

        Page<ProductionOrderVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ProductionOrderVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        //赋值订单明细信息
        List<Long> produceOrderIds = records.stream().map(ProductionOrder::getId).collect(Collectors.toList());
        List<ProductionOrderDetail> productionOrderDetailList = produceOrderDetailService.list(q -> q.in(ProductionOrderDetail::getProduceOrderId, produceOrderIds));
        List<ProductionOrderDetailVo> produceOrderDetailVos = BeanUtil.copyToList(productionOrderDetailList, ProductionOrderDetailVo.class);

        productInfoService.attributeAssign(produceOrderDetailVos, ProductionOrderDetail::getProductId, (item, product) -> {
            item.setProductCode(product.getCustomCode());
            item.setProductName(product.getName());
            item.setProductUnit(product.getUnit());
            item.setProductSpec(product.getSpec());
            item.setProductLength(product.getLength());
            item.setProductWidth(product.getWidth());
            item.setProductHeight(product.getHeight());
        });

        Map<Long, List<ProductionOrderDetailVo>> produceOrderDetailMap = produceOrderDetailVos.stream().collect(Collectors.groupingBy(ProductionOrderDetail::getProduceOrderId));
        for (ProductionOrderVo record : records) {
            record.setProduceOrderDetailList(produceOrderDetailMap.get(record.getId()));
        }

        return page;
    }

    @Override
    @DSTransactional
    public void createOrder(ProductionOrderDto produceOrderDto) {
        Long contractId = produceOrderDto.getContractId();
        Assert.notEmpty(produceOrderDto.getContractId(), "合同id不能为空！");

        //将数据移到归属公司下
        SysDept produceCompany = sysDeptService.getById(produceOrderDto.getProduceCompanyId());
        Assert.notEmpty(produceCompany, "查询不到生产公司信息");
        String tenantId = produceCompany.getTenantId();
        produceOrderDto.setTenantId(tenantId);

        //获取合同信息
        Contract contract = contractService.getById(contractId);
        Assert.notEmpty(contract, "查询不到合同信息！");
        contractService.update(q -> q.eq(Contract::getId, contractId).set(Contract::getOrderDistributeStatus, 1));

        produceOrderDto.setCode(contract.getCode());
        produceOrderDto.setProduceStatus(0);
        this.save(produceOrderDto);

        //创建生产订单明细
        List<ProductionOrderDetail> productionOrderDetailList = new ArrayList<>();
        List<ContractProduct> contractProductList = contractProductService.list(q -> q.eq(ContractProduct::getContractId, contractId));

        //获取产品id
        List<Long> productIds = contractProductList.stream().map(ContractProduct::getProductId).collect(Collectors.toList());
        Map<Long, ProductInfo> productInfoMap = productInfoService.mapKEntity(ProductInfo::getId, q -> q.in(ProductInfo::getId, productIds));
        Map<Long, List<ProductBomDetail>> productBomDetailMap = productBomDetailService.mapKGroup(ProductBomDetail::getProductId, q -> q.in(ProductBomDetail::getProductId, productIds));

        for (ContractProduct contractProduct : contractProductList) {
            ProductionOrderDetail productionOrderDetail = new ProductionOrderDetail();
            productionOrderDetail.setProduceOrderId(produceOrderDto.getId());
            productionOrderDetail.setContractId(contractId);
            productionOrderDetail.setContractDetailId(contractProduct.getId());
            productionOrderDetail.setProductId(contractProduct.getProductId());
            productionOrderDetail.setQuantity(contractProduct.getQuantity());
            productionOrderDetail.setFinishQuantity(BigDecimal.ZERO);
            productionOrderDetail.setProduceStatus(0);
            productionOrderDetail.setTenantId(tenantId);
            productionOrderDetailList.add(productionOrderDetail);

            //创建备料
            Long productId = contractProduct.getProductId();
            ProductInfo productInfo = productInfoMap.get(productId);
            Assert.notEmpty(productInfo, "查询不到产品信息");

//            //创建原材料
//            ProductBomDetail productBomDetail = new ProductBomDetail();
//            productBomDetail.setProductId(productInfo.getId());
//            productBomDetail.setMaterialId(productInfo.getRawMaterialId());
//            productBomDetail.setQuantity(BigDecimal.ONE);
            List<ProductBomDetail> productBomDetailList = productBomDetailMap.get(productId);
//            productBomDetailList.add(productBomDetail);

            List<MaterialPreparation> materialPreparationList = new ArrayList<>();
            for (ProductBomDetail bomDetail : productBomDetailList) {
                BigDecimal multiply = bomDetail.getQuantity().multiply(contractProduct.getQuantity());

                MaterialPreparation materialPreparation = new MaterialPreparation();
                materialPreparation.setContractId(contractId);
                materialPreparation.setContractDetailId(contractProduct.getId());
                materialPreparation.setProductionOrderId(produceOrderDto.getId());
                materialPreparation.setProductionOrderDetailId(productionOrderDetail.getId());
                materialPreparation.setMaterialId(bomDetail.getMaterialId());
                materialPreparation.setQuantity(multiply);
                materialPreparation.setStatus(0);
                materialPreparationList.add(materialPreparation);
            }
            materialPreparationService.saveBatch(materialPreparationList);
        }
        produceOrderDetailService.saveBatch(productionOrderDetailList);

        //创建生产订单明细
        int index = 0;
        List<WorkOrder> workOrderList = new ArrayList<>();
        for (ContractProduct contractProduct : contractProductList) {
            index++;
            //生成工单
            for (int i = 1; i <= contractProduct.getQuantity().intValue(); i++) {
                WorkOrder workOrder = new WorkOrder();
                workOrder.setCode(contract.getCode() + "-" + index + "-" + i);
                workOrder.setProductId(contractProduct.getProductId());
                workOrder.setQuantity(BigDecimal.ONE);
                workOrder.setStatus(0);
                workOrder.setContractId(contractId);
                workOrder.setContractDetailsId(contractProduct.getId());
                workOrderList.add(workOrder);
            }
        }
        workOrderService.saveBatch(workOrderList);
    }

}
